<?php

namespace Drupal\course_credit\Entity;

use Drupal\Core\Entity\EditorialContentEntityBase;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Field\BaseFieldDefinition;

/**
 * Defines the course credit type entity class.
 *
 * @ContentEntityType(
 *   id = "course_credit",
 *   label = @Translation("Course credit"),
 *   label_collection = @Translation("Course credit"),
 *   label_singular = @Translation("course credit"),
 *   label_plural = @Translation("course credit"),
 *   label_count = @PluralTranslation(
 *     singular = "@count course credit",
 *     plural = "@count course credit",
 *   ),
 *   admin_permission = "administer course credit",
 *   base_table = "course_credit",
 *   fieldable = FALSE,
 *   show_revision_ui = TRUE,
 *   revision_table = "course_credit_revision",
 *   entity_keys = {
 *     "id" = "ccid",
 *     "revision" = "vid",
 *     "published" = "active",
 *     "uuid" = "uuid",
 *     "bundle" = "type",
 *   },
 *   bundle_label = @Translation("Course credit type"),
 *   bundle_entity_type = "course_credit_type",
 *   revision_metadata_keys = {
 *     "revision_default" = "revision_default",
 *     "revision_user" = "revision_uid",
 *     "revision_created" = "revision_timestamp",
 *     "revision_log_message" = "revision_log",
 *   },
 *   handlers = {
 *     "list_builder" = "Drupal\Core\Entity\EntityListBuilder",
 *     "route_provider" = {
 *       "html" = "Drupal\Core\Entity\Routing\AdminHtmlRouteProvider",
 *     },
 *     "access" = "Drupal\entity\EntityAccessControlHandler",
 *     "permission_provider" = "Drupal\entity\EntityPermissionProvider",
 *     "form" = {
 *       "default" = "Drupal\Core\Entity\ContentEntityForm",
 *       "delete" = "Drupal\Core\Entity\EntityDeleteForm",
 *     },
 *     "views_data" = "Drupal\entity\EntityViewsData",
 *   },
 *   links = {
 *     "collection" = "/admin/course/credit/instance",
 *   }
 * )
 */
class CourseCredit extends EditorialContentEntityBase {

  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
    $fields = parent::baseFieldDefinitions($entity_type);

    $fields['type']
      ->setDisplayOptions('view', [
        'region' => 'content',
        'label' => 'visually_hidden',
        'type' => 'entity_reference_label',
        'settings' => ['link' => FALSE],
        'weight' => 1,
      ]);

    $fields['changed'] = BaseFieldDefinition::create('changed')
      ->setRevisionable(TRUE)
      ->setLabel('Changed');

    $fields['children'] = BaseFieldDefinition::create('entity_reference')
      ->setRevisionable(TRUE)
      ->setCardinality(-1)
      ->setLabel('Child credits')
      ->setDisplayOptions('form', [
        'type' => 'inline_entity_form_complex',
      ])
      ->setSetting('target_type', 'course_credit');

    $fields['min'] = BaseFieldDefinition::create('decimal')
      ->setDefaultValue(0)
      ->setLabel('Minimum')
      ->setRevisionable(TRUE)
      ->setDisplayOptions('view', [
        'region' => 'content',
        'type' => 'number_decimal',
        'weight' => 0,
      ])
      ->setDisplayOptions('form', [
        'type' => 'number',
      ]);

    $fields['increments'] = BaseFieldDefinition::create('decimal')
      ->setDefaultValue(0)
      ->setLabel('Increments')
      ->setRevisionable(TRUE)
      ->setDisplayOptions('view', [
        'type' => 'number_decimal',
      ])
      ->setDisplayOptions('form', [
        'type' => 'number',
      ]);

    $fields['max'] = BaseFieldDefinition::create('decimal')
      ->setDefaultValue(0)
      ->setLabel('Maximum')
      ->setRevisionable(TRUE)
      ->setDisplayOptions('view', [
        'type' => 'number_decimal',
      ])
      ->setDisplayOptions('form', [
        'type' => 'number',
      ]);
    $fields['variable'] = BaseFieldDefinition::create('boolean')
      ->setLabel('Variable')
      ->setRevisionable(TRUE)
      ->setDisplayOptions('form', [
        'type' => 'boolean_checkbox',
      ]);
    $fields['active'] = BaseFieldDefinition::create('boolean')
      ->setLabel('Active')
      ->setRevisionable(TRUE)
      ->setDisplayOptions('form', [
        'type' => 'boolean_checkbox',
      ]);
    $fields['code'] = BaseFieldDefinition::create('string')
      ->setLabel('Code')
      ->setRevisionable(TRUE)
      ->setDisplayOptions('form', [
        'type' => 'string_textfield',
      ]);

    $fields['expiration_type'] = BaseFieldDefinition::create('list_string')
      ->setLabel('Expiration type')
      ->setDisplayOptions('form', [
        'type' => 'options_select',
      ])
      ->setSetting('allowed_values', [
        'date' => 'Date',
        'calendar_year' => 'Calendar year',
        'credit_awarded' => 'Credit awarded',
      ])
      ->setRevisionable(TRUE);

    $fields['expiration_date'] = BaseFieldDefinition::create('datetime')
      ->setLabel('Expiration date')
      ->setRevisionable(TRUE)
      ->setDisplayOptions('form', [
        'type' => 'datetime_default',
      ]);

    $fields['expiration_offset'] = BaseFieldDefinition::create('integer')
      ->setLabel('Plus additional time')
      ->setDescription('For relative types, optionally specify an offset from the selected type.')
      ->setRevisionable(TRUE);

    $fields['expiration_offset_duration'] = BaseFieldDefinition::create('duration')
      ->setLabel('Plus additional time')
      ->setDescription('For relative types, optionally specify an offset from the selected type.')
      ->setSetting('granularity', 'd:h:i')
      ->setDisplayOptions('form', [
        'type' => 'duration_widget',

      ])
      ->setRevisionable(TRUE);

    return $fields;
  }

  /**
   * Label the credit instance.
   *
   * {@inheritdoc}
   */
  function label() {
    $bundle = $this->get('type')->referencedEntities()[0]->label();
    $max = $this->get('max')->value;
    return \Drupal::translation()->formatPlural($max, "1.00 $bundle credit", "@count $bundle credits");
  }

  /**
   * Get the credit type.
   *
   * @return CourseCreditType
   */
  function getType() {
    return $this->get('type')->referencedEntities()[0];
  }

}
