<?php

namespace Drupal\course_credit\Entity;

use Drupal;
use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EditorialContentEntityBase;
use Drupal\Core\Entity\EntityChangedTrait;
use Drupal\Core\Entity\EntityMalformedException;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\course\Entity\CourseEnrollment;
use Drupal\datetime\Plugin\Field\FieldType\DateTimeItemInterface;
use Drupal\field\Entity\FieldStorageConfig;


/**
 * Defines the course credit type entity class.
 *
 * @ContentEntityType(
 *   id = "course_credit_awarded",
 *   label = @Translation("Course credit awarded"),
 *   label_collection = @Translation("Course credit awarded"),
 *   label_singular = @Translation("course credit awarded"),
 *   label_plural = @Translation("course credit awarded"),
 *   label_count = @PluralTranslation(
 *     singular = "@count course credit awarded",
 *     plural = "@count course credit awarded",
 *   ),
 *   admin_permission = "administer course credit",
 *   base_table = "course_credit_awarded",
 *   fieldable = TRUE,
 *   field_ui_base_route = "entity.course_credit_awarded.collection",
 *   show_revision_ui = TRUE,
 *   revision_table = "course_credit_awarded_revision",
 *   revision_data_table = "course_credit_awarded_field_revision",
 *   entity_keys = {
 *     "id" = "ccaid",
 *     "revision" = "vid",
 *     "published" = "status",
 *     "uuid" = "uuid",
 *   },
 *   revision_metadata_keys = {
 *     "revision_default" = "revision_default",
 *     "revision_user" = "revision_uid",
 *     "revision_created" = "revision_timestamp",
 *     "revision_log_message" = "revision_log",
 *   },
 *   handlers = {
 *     "list_builder" = "Drupal\Core\Entity\EntityListBuilder",
 *     "route_provider" = {
 *       "html" = "Drupal\Core\Entity\Routing\AdminHtmlRouteProvider",
 *     },
 *    "form" = {
 *       "default" = "Drupal\Core\Entity\ContentEntityForm",
 *       "delete" = "Drupal\Core\Entity\EntityDeleteForm",
 *     },
 *     "views_data" = "Drupal\entity\EntityViewsData",
 *   },
 *   links = {
 *     "canonical" = "/course-credit-awarded/{course_credit_awarded}",
 *     "collection" = "/admin/course/credit/awarded"
 *   }
 * )
 */
class CourseCreditAwarded extends EditorialContentEntityBase {

  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
    $fields = parent::baseFieldDefinitions($entity_type);

    $fields['credits'] = BaseFieldDefinition::create('decimal')
      ->setRevisionable(TRUE)
      ->setDisplayConfigurable('view', TRUE)
      ->setLabel(t('Credits'));

    $fields['type'] = BaseFieldDefinition::create('entity_reference')
      ->setSetting('target_type', 'course_credit_type')
      ->setRevisionable(TRUE)
      ->setDisplayConfigurable('view', TRUE)
      ->setLabel(t('Credit type'));

    $fields['date'] = BaseFieldDefinition::create('created')
      ->setRevisionable(TRUE)
      ->setLabel('Date of award');

    $fields['changed'] = BaseFieldDefinition::create('changed')
      ->setRevisionable(TRUE)
      ->setLabel('Changed');

    $fields['expiration'] = BaseFieldDefinition::create('datetime')
      ->setLabel('Date credit expires');

    $fields['children'] = BaseFieldDefinition::create('entity_reference')
      ->setCardinality(-1)
      ->setLabel('Child credits awarded')
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('form', [
        'type' => 'inline_entity_form_complex',
      ])
      ->setSetting('target_type', 'course_credit_awarded');

    $fields['parent_id'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Parent ID'))
      ->setDescription(t('The ID of the parent entity of which this entity is referenced.'))
      ->setSetting('is_ascii', TRUE)
      ->setRevisionable(TRUE);

    $fields['parent_type'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Parent type'))
      ->setDescription(t('The entity parent type to which this entity is referenced.'))
      ->setSetting('is_ascii', TRUE)
      ->setSetting('max_length', EntityTypeInterface::ID_MAX_LENGTH)
      ->setRevisionable(TRUE);

    $fields['parent_field_name'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Parent field name'))
      ->setDescription(t('The entity parent field name to which this entity is referenced.'))
      ->setSetting('is_ascii', TRUE)
      ->setSetting('max_length', FieldStorageConfig::NAME_MAX_LENGTH)
      ->setRevisionable(TRUE);

    return $fields;
  }

  /**
   * Gets the parent entity of the credit.
   *
   * @return ContentEntityInterface
   *   The parent entity.
   */
  public function getParentEntity() {
    if (!isset($this->get('parent_type')->value) || !isset($this->get('parent_id')->value)) {
      return NULL;
    }

    $parent = \Drupal::entityTypeManager()->getStorage($this->get('parent_type')->value)->load($this->get('parent_id')->value);

    return $parent;
  }

  /**
   * Set the parent entity of the credit.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $parent
   *   The parent entity.
   * @param string $parent_field_name
   *   The parent field name.
   *
   * @return $this
   */
  public function setParentEntity(ContentEntityInterface $parent, $parent_field_name) {
    $this->set('parent_type', $parent->getEntityTypeId());
    $this->set('parent_id', $parent->id());
    $this->set('parent_field_name', $parent_field_name);
    return $this;
  }

  /**
   * {@inheritdoc}
   *
   * We need a parent entity.
   */
  public function preSave(Drupal\Core\Entity\EntityStorageInterface $storage) {
    if (!$this->getParentEntity()) {
      throw new EntityMalformedException('Credit awarded parent not set.');
    }

    parent::preSave($storage);
  }

}
