<?php

namespace Drupal\course_credit\Form;

use Drupal;
use Drupal\Component\Utility\Html;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\course\Entity\Course;
use Drupal\course_credit\Entity\CourseCreditType;
use function _course_credit_app_validate;
use function _course_credit_claim_credit;
use function _course_credit_claimed_params;
use function course_credit_user_credit_types;

class CourseCreditAppForm extends FormBase {

  public function buildForm(array $form, FormStateInterface $form_state, Course $course = NULL) {
    $config = $this->config('course_credit.settings');
    $user = Drupal::currentUser();
    $course_enrollment = $course->getEnrollment($user);
    $t = Drupal::translation();
    $form['#title'] = t('@title: Credit claiming', ['@title' => $course->label()]);

    // Claim credit for credit types.
    $form['credits_claimed'] = array(
      '#tree' => TRUE,
    );

    $credits = course_credit_user_credit_types($course_enrollment, $course, $user);

    // Is the user claiming variable credit?
    $claiming = FALSE;

    foreach ($credits as $credit) {
      // Get additional type info (type & title).
      $type = $credit->getType()->id();
      $credit_type = CourseCreditType::load($type);
      $label = $credit_type->label();
      $en_var = $credit->get('variable')->getString();
      $max = $credit->get('max')->getString();
      $min = $credit->get('min')->getString();
      $inc = $credit->get('increments')->getString();
      //$sub_credits = $credit_type->data['sub_credits'];

      $form['credits_claimed'][$type] = array(
        '#type' => 'fieldset',
        '#title' => $label,
        '#attributes' => array('class' => array('course-credit-claimed-type')),
      );
      $form['credits_claimed'][$type]['claim'] = array(
        '#title' => $label,
        '#type' => 'checkbox',
        '#default_value' => $config->get('select_all'),
      );

      $string = "edit-credits-claimed-{$type}-claim";
      $html_id = '#' . preg_replace('/\-+/', '-', Html::cleanCssIdentifier($string));
      $states = array(
        'visible' => array(
          $html_id => array('checked' => TRUE),
        ),
      );
      $form['credits_claimed'][$type]['credits'] = array(
        '#type' => 'container',
        '#states' => $states,
      );

      if ($en_var) {
        // Variable credit enabled.
        $opts = array(
          '@title' => $label,
          '@credits' => number_format($max, 2),
          '@plural' => $t->formatPlural($max, 'credits', 'credits'),
          '@increments' => $inc,
        );
        $description = t('<em>@title can be claimed in @increments increments up to a maximum of @credits.</em><br>Enter the amount of credits to claim.', $opts);
        $form['credits_claimed'][$type]['credits']['amount'] = array(
          '#type' => $inc > 0 ? 'number' : 'textfield',
          '#step' => $inc,
          '#min' => $min,
          '#max' => $max,
          '#title' => t('Credits'),
          '#title_display' => 'invisible',
          '#default_value' => NULL,
          '#size' => 6,
          '#description' => $description,
        );

        // Now we prevents skipping the credit claim screen from the router.
        $claiming = 1;
      }
      else {
        // Variable credit not enabled.
        $opts = array(
          '@title' => $label,
          '@credits' => number_format($max, 2),
          '@plural' => $t->formatPlural($max, 'credit', 'credits'),
        );
        $description = t('<em>You can claim @credits @title.</em> This cannot be changed.', $opts);
        $form['credits_claimed'][$type]['credits']['amount'] = array(
          '#type' => 'textfield',
          '#title' => t('Credits'),
          '#title_display' => 'invisible',
          '#default_value' => number_format($max, 2),
          '#disabled' => TRUE,
          '#description' => $description,
          '#value' => number_format($max, 2),
          '#size' => 4,
        );
      }

      $sub_credits = $credit->get('children')->referencedEntities();

      // Handle child credits
      if (!empty($sub_credits)) {
        $parent_key = $type;
        $form['credits_claimed'][$parent_key]['credits']['child_credits'] = array(
          '#title' => t('Additional credits available'),
          '#type' => 'fieldset',
          '#attributes' => array(
            'class' => array(
              Html::cleanCssIdentifier("credit-type-{$parent_key}-child"),
              "child-credit-type",
            ),
          ),
        );
        foreach ($sub_credits as $key => $sub_credit) {
          $sub_type = CourseCreditType::load($sub_credit->getType()->id());
          $form['credits_claimed'][$parent_key]['credits']['child_credits'][$sub_credit->getType()->id()]['claim'] = array(
            '#title' => $sub_type->label(),
            '#type' => 'checkbox',
            '#default_value' => $config->get('select_all'),
          );
          $string = "edit-credits-claimed-$parent_key-credits-child-credits-{$sub_credit->getType()->id()}-claim";
          $html_id = '#' . preg_replace('/\-+/', '-', Html::cleanCssIdentifier($string));
          $states = array(
            'visible' => array(
              $html_id => array('checked' => TRUE),
            ),
          );
          $form['credits_claimed'][$parent_key]['credits']['child_credits'][$sub_credit->getType()->id()]['credits'] = array(
            '#type' => 'container',
            '#states' => $states,
          );
          $opts = array(
            '@title' => $sub_type->label(),
            '@credits' => number_format($sub_credit->get('max')->value, 2),
            '@increments' => number_format($sub_credit->get('increments')->value, 2),
            '@plural' => \Drupal::translation()->formatPlural($sub_credit->get('max')->value, 'credit', 'credits'),
          );
          if ($sub_credit->get('variable')->value) {
            // Variable credit enabled.
            $description = t('<em>@title can be claimed in @increments increments up to a maximum of @credits.</em><br>Enter the amount of credits to claim.', $opts);
            $form['credits_claimed'][$parent_key]['credits']['child_credits'][$sub_credit->getType()->id()]['credits']['amount'] = array(
              '#type' => 'number',
              '#title' => $sub_type->label(),
              '#title_display' => 'invisible',
              '#default_value' => NULL,
              '#size' => 4,
              '#description' => $description,
              '#states' => array(
                'disabled' => array(
                  $html_id => array('checked' => FALSE),
                ),
              ),
              '#min' => $sub_credit->get('min')->value,
              '#max' => $sub_credit->get('max')->value,
              '#step' => $sub_credit->get('increments')->value,
            );
            // Now we prevents skipping the credit claim screen from the router.
            $claiming = 1;
          }
          else {
            // Variable credit not enabled.
            $description = t('<em>You can claim @credits @title.</em> This cannot be changed.', $opts);
            $form['credits_claimed'][$parent_key]['credits']['child_credits'][$sub_credit->getType()->id()]['credits']['amount'] = array(
              '#type' => 'textfield',
              '#title' => $sub_credit->label(),
              '#title_display' => 'invisible',
              '#default_value' => $sub_credit->max->value,
              '#size' => 4,
              '#disabled' => TRUE,
              '#description' => $description,
              '#value' => $sub_credit->max->value,
            );
          }
        }
      }
    }

    $form['not_claiming'] = array(
      '#type' => 'checkbox',
      '#title' => t('I am not claiming any credit at this time.'),
      '#access' => $config->get('allow_skip'),
    );

    foreach (array_keys($credits) as $key) {
      $html_id = '#' . preg_replace('/\-+/', '-', Html::cleanCssIdentifier('edit-credits-claimed-' . $key . '-claim'));
      $form['not_claiming']['#states']['visible'][$html_id] = array('checked' => FALSE);
      $form['not_claiming']['#states']['required'][$html_id] = array('checked' => FALSE);
    }

    $form['agreement'] = array(
      '#type' => 'checkbox',
      '#title' => t('I agree that I am only claiming credit commensurate with the extent of my participation in the activity.'),
      // Do not show agreement if user is claiming max automatically.
      '#access' => $claiming == 1,
      '#weight' => 0,
    );

    foreach (array_keys($credits) as $key) {
      $html_id = '#' . preg_replace('/\-+/', '-', Html::cleanCssIdentifier('edit-credits-claimed-' . $key . '-claim'));
      $form['agreement']['#states']['visible'][] = array($html_id => array('checked' => TRUE));
      $form['agreement']['#states']['required'][] = array($html_id => array('checked' => TRUE));
    }

    $current_path = \Drupal::service('path.current')->getPath();
    $path_args = explode('/', $current_path);
    if ($path_args[2] == 'inframe') {
      $form['inframe'] = array(
        '#type' => 'hidden',
        '#value' => 1,
      );
    }

    if (empty($credits)) {
      $form['credits_claimed']['no_credit']['#markup'] = t('There is no valid credit to be awarded for this activity.');
    }
    else {
      $form['submit'] = array(
        '#type' => 'submit',
        '#value' => t('Submit'),
        '#weight' => 100,
      );
    }

    return $form;
  }

  public function getFormId(): string {
    return 'course_credit_app_form';
  }

  public function submitForm(array &$form, FormStateInterface $form_state) {
    $course = $form_state->getBuildInfo()['args'][0];
    $account = Drupal::currentUser();
    $form_post = $form_state->getValues();
    $credits_claimed = $form_post['credits_claimed'];

    if (is_array($credits_claimed)) {
      foreach ($credits_claimed as $credit_type_name => $credits_value) {
        if ($credits_value['claim']) {
          _course_credit_claim_credit($course, $account, $credit_type_name, $credits_value);
        }
      }
    }

    $current_path = \Drupal::service('path.current')->getPath();
    $path_args = explode('/', ltrim($current_path, '/'));
    if ($path_args[2] == 'course-credit-app') {
      // KLUDGE: we don't redirect unless we are outside of the course object.
      // arg(2) likely not the best way here.
      $form_state->setRedirect('course.complete', ['course' => $course->id()]);
    }
  }

  /**
   * Validate the credit claiming form.
   */
  function validateForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('course_credit.settings');
    parent::validateForm($form, $form_state);

    $user = $this->currentUser();
    $values = $form_state->getValues();

    // Validate credits claimed.
    // Get course node.
    $credits_claimed = $values['credits_claimed'];

    /* @var \Drupal\course\Entity\Course $course */
    $course = $form_state->getBuildInfo()['args'][0];

    $course_enrollment = $course->getEnrollment($user);

    $credit_types = course_credit_user_credit_types($course_enrollment, $course, $user);

    // How many types the user is claiming.
    $claiming = 0;

    // Alter the eligible types.
    $context = [
      'course' => $course,
      'account' => $user,
    ];
    Drupal::moduleHandler()->alter('course_credit_user_credit_types', $valid_credit_instances, $parent, $context);

    $credits_claimed_params = array();
    foreach ($credit_types as $credit_type) {
      _course_credit_claimed_params($credits_claimed_params, $credit_type);
    }

    if (is_array($credits_claimed)) {
      foreach ($credits_claimed as $key => $claim_set) {
        $element = $form['credits_claimed'][$key]['credits'];
        _course_credit_app_validate($form, $form_state, $claiming, $element, $credits_claimed_params, $key, $claim_set);
      }
    }

    if (!$claiming && !$config->get('allow_skip')) {
      $form_state->setErrorByName('', t('Please claim a type.'));
    }

    if ($claiming && $form['agreement']['#access']) {
      if (empty($values['agreement'])) {
        $form_state->setErrorByName('agreement', t('Please verify your statement.'));
      }
    }

    if (!$claiming && $form['not_claiming']['#access']) {
      if (empty($values['not_claiming'])) {
        $form_state->setErrorByName('not_claiming', t('Please indicate that you are not claiming any credit.'));
      }
    }

    // You cannot claim more child credit than parent credit.
    foreach ($credits_claimed as $parentkey => $parent) {
      if ($parent['claim'] && !empty($parent['credits']['child_credits'])) {
        foreach ($parent['credits']['child_credits'] as $childkey => $child) {
          if ($child['claim']) {
            if ($child['credits']['amount'] > $parent['credits']['amount']) {
              $names = ['%parent' => $form['credits_claimed'][$parentkey]['#title'], '%child' => $form['credits_claimed'][$parentkey]['credits']['child_credits'][$childkey]['claim']['#title']];
              //credits_claimed[parental][credits][child_credits][children][credits][amount]
              $path = "credits_claimed][$parentkey][credits][child_credits][$childkey][credits][amount";
              $form_state->setErrorByName($path, t('%child credit amount cannot exceed %parent claimed credit amount', $names));
            }
          }
        }
      }
    }
  }

}
