<?php

namespace Drupal\course_credit\Plugin\course\CourseObject;

use Drupal;
use Drupal\Component\Utility\Html;
use Drupal\Component\Utility\Xss;
use Drupal\Core\Form\FormStateInterface;
use Drupal\course\Entity\CourseObject;
use Drupal\course_credit\Entity\CourseCreditType;
use function course_credit_award_credit;
use function course_credit_must_claim;

/**
 * @CourseObject(
 *   id = "credit",
 *   label = "Credit",
 *   handlers = {
 *     "fulfillment" = "\Drupal\course\Entity\CourseObjectFulfillment"
 *   }
 * )
 */
class CourseObjectCredit extends CourseObject {

  public function defaultConfiguration() {
    $defaults = parent::defaultConfiguration();
    $defaults['required'] = 0;
    $defaults['credit_types'] = [];
    return $defaults;
  }

  public function take() {
    $t = Drupal::translation();
    $user = Drupal::currentUser();
    $course = $this->getCourse();
    $course_enrollment = $course->getEnrollment($user);
    $awarded = $course_enrollment->get('course_credit_awarded')->referencedEntities();
    $content = [
      '#markup' => t('Sorry, there is no credit to be awarded.'),
    ];
    if (!$awarded) {
      // No credit awarded yet, let's try to award credit.

      if (course_credit_must_claim($course_enrollment, $course, $user)) {
        // User must claim variable credit, so show them the claiming form.
        $form = \Drupal::formBuilder()->getForm(\Drupal\course_credit\Form\CourseCreditAppForm::class, $course);
        return $form;
      }

      // There wasn't any variable credit to be awarded so let's try to claim
      // nonvariable credit.
      course_credit_award_credit($course_enrollment, $course, $user);
    }

    // Get credit again to see if anything changed.
    $awarded = $course_enrollment->get('course_credit_awarded')->referencedEntities();
    if ($awarded) {
      $items = [];
      foreach ($awarded as $credit_awarded) {
        $type = $credit_awarded->get('type')->getString();
        $credit_type = CourseCreditType::load($type);
        $credit_amount = $credit_awarded->get('credits')->getString();
        $item = t('You were awarded <span class="course-credit-awarded-credit">@credits</span> <span class="course-credit-awarded-title">@title</span> @creditp.', array(
          '@credits' => number_format($credit_amount, 2),
          '@title' => Xss::filterAdmin($credit_type->label()),
          '@creditp' => $t->formatPlural($credit_amount, 'credit', 'credits')
        ));

        $items[$type] = [
          '#markup' => $item,
          '#wrapper_attributes' => [
            'class' => ['course-credit-awarded-' . Html::cleanCssIdentifier($type)],
          ],
        ];

        // Process child credits.
        foreach ($credit_awarded->get('children')->referencedEntities() as $sub_credit_awarded) {
          $type = $sub_credit_awarded->get('type')->getString();
          $credit_type = CourseCreditType::load($type);
          $credit_amount = $sub_credit_awarded->get('credits')->getString();
          $item = t('You were awarded <span class="course-credit-awarded-credit">@credits</span> <span class="course-credit-awarded-title">@title</span> @creditp.', array(
            '@credits' => number_format($credit_amount, 2),
            '@title' => Xss::filterAdmin($credit_type->label()),
            '@creditp' => $t->formatPlural($credit_amount, 'credit', 'credits')
          ));

          $items[$type]['children'][] = $item;
        }
      }
      $content = [
        '#theme' => 'item_list',
        '#list_type' => 'ul',
        '#items' => $items,
        '#attributes' => ['class' => 'course-credit-awarded-list'],
      ];
    }

    return $content;
  }

  function save() {
    $this->setInstanceId($this->get('cid')->getString());
    return parent::save();
  }

  function getCloneAbility() {
    return TRUE;
  }

  function buildConfigurationForm($form, FormStateInterface $form_state) {
    $form += parent::buildConfigurationForm($form, $form_state);

    $config = $this->getConfiguration();

    // Get active instances on the course.
    $types = [];
    foreach ($this->getCourse()->get('credit')->referencedEntities() as $credit_instance) {
      if ($credit_instance->get('active')->value) {
        $types[$credit_instance->getType()->id()] = $credit_instance->getType()->label();
      }
    }

    $form['credit_types'] = [
      '#title' => t('Credit types'),
      '#description' => t('If the credit types selected below are variable, they will be available for claiming on this course object. If no credit types are selected here, all will be available. Regardless of what is selected here, users must be eligible for the credit type in order for it to be awarded. <strong>Non-variable credit will always be awarded and ignore any selections below.</strong>'),
      '#type' => 'checkboxes',
      '#default_value' => $config['credit_types'],
      '#options' => $types,
      '#access' => !empty($types),
    ];

    return $form;
  }

}
