<?php

namespace Drupal\course_credit\Plugin\views\field;

use Drupal;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Component\Utility\Html;
use Drupal\course\Entity\Course;
use Drupal\course_credit\Entity\CourseCredit;
use Drupal\views\Plugin\views\display\DisplayPluginBase;
use Drupal\views\Plugin\views\field\PrerenderList;
use Drupal\views\ViewExecutable;

/**
 * A handler to provide a field that pulls the active credit types.
 *
 * @ingroup views_field_handlers
 *
 * @ViewsField("course_credit_active_types")
 */
class CourseCreditActiveTypes extends PrerenderList {

  /**
   * {@inheritdoc}
   */
  public function init(ViewExecutable $view, DisplayPluginBase $display, array &$options = NULL) {
    parent::init($view, $display, $options);

    $this->additional_fields['cid'] = [
      'table' => 'course',
      'field' => 'cid',
    ];
  }

  /**
   * Add this term to the query
   */
  public function query() {
    $this->addAdditionalFields();
    $this->field_alias = $this->aliases['cid'];
  }

  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    $allowed_types = course_credit_get_credit_types(1);

    $options = [];
    if ($allowed_types) {
      foreach ($allowed_types as $ctid => $type) {
        $options[$ctid] = $type->label();
      }
    }

    $form['course_credit_types'] = array(
      '#title' => 'Credit types',
      '#type' => 'checkboxes',
      '#options' => $options,
      '#description' => 'Select credit types to show instead of showing all types.',
      '#default_value' => $this->options['course_credit_types'] ? $this->options['course_credit_types'] : array(),
    );

    $form['course_credit_types_negate'] = array(
      '#title' => 'Negate',
      '#type' => 'checkbox',
      '#description' => 'Do not show selected types.',
      '#default_value' => $this->options['course_credit_types_negate'],
    );

    $form['course_credit_types_format'] = array(
      '#title' => 'Format',
      '#type' => 'textfield',
      '#description' => 'Specify a format to use other than the name of the active credit type. Possible tokens: [title] [type] [min] [max]. You can also use [min-s] or [max-s] for the "S" after "credit" based on the number of credits.',
      '#default_value' => $this->options['course_credit_types_format'],
    );

    parent::buildOptionsForm($form, $form_state);
  }

  protected function defineOptions() {
    $options = parent::defineOptions();

    $options['course_credit_types'] = array('default' => array());
    $options['course_credit_types_negate'] = array('default' => FALSE);
    $options['course_credit_types_format'] = array('default' => '');

    return $options;
  }

  /**
   * Get the active credit types on each of the courses in this view.
   */
  public function preRender(&$values) {
    $cids = array();
    $cid_field = $this->field_alias;

    foreach ($values as $value) {
      $cids[] = $value->{$cid_field};
    }

    if (empty($cids)) {
      return;
    }

    $system_credit_types = course_credit_get_credit_types();

    // Selected credit types from the views field config form.
    if (!empty($this->options['course_credit_types'])) {
      $selected = array_filter($this->options['course_credit_types']);
    }

    if (empty($selected)) {
      $selected = array_keys($system_credit_types);
      $selected = array_flip($selected);

      if (empty($selected)) {
        // There are no credit types.
        return;
      }
    }

    // Get active instances on the course.
    $allowed_types = [];
    $credit_instances = [];

    foreach ($cids as $cid) {
      $course = Course::load($cid);
      $credits_all = $this->getAllCredits($course);
      foreach ($credits_all as $credit) {
        // Only active credit.
        $is_active = $credit->get('active')->getValue()[0]['value'];
        if (empty($is_active)) {
          continue;
        }
        $allowed_types[$credit->getType()->id()] = $credit->getType()->label();
        $credit_instances[$credit->id()] = $credit;
      }

      // Exclude credit types in the selected list.
      if (!empty($this->options['course_credit_types_negate'])) {
        $allowed_types = array_diff_key($allowed_types, $selected);
      }
      // Include only credit types in the selected list.
      else {
        $allowed_types = array_intersect_key($allowed_types, $selected);
      }
    }

    if (empty($allowed_types)) {
      return;
    }

    $items = [];
    foreach ($credit_instances as $ccid => $course_credit) {
      // Filter out credit based on the views field config.
      $ctype = $course_credit->get('type')->getValue()[0]['target_id'];
      if (!in_array($ctype, array_keys($allowed_types))) {
        continue;
      }
      // Filter out child credit, those will be handled in the children array.
      $is_child = $system_credit_types[$ctype]->get('sub');
      if ($is_child) {
        continue;
      }
      $courses = Drupal::entityTypeManager()->getStorage('course')->loadByProperties([
        'credit' => $ccid,
      ]);
      $course = reset($courses);
      $cid = $course->id();

      $items[$cid][0][$ccid] = $this->buildCreditValues($course_credit, $ctype, $allowed_types);

      $children = $course_credit->get('children')->referencedEntities();
      $items[$cid][0][$ccid]['sub_credits'] = [];
      if (!empty($children)) {
        foreach ($children as $child) {
          // Filter out credit based on the views field config.
          $ctype = $child->get('type')->getValue()[0]['target_id'];
          if (!in_array($ctype, array_keys($allowed_types))) {
            continue;
          }
          $is_active = $child->get('active')->getValue()[0]['value'];
          if ($is_active) {
            $items[$cid][0][$ccid]['sub_credits'][$ctype] = $this->buildCreditValues($child, $ctype, $allowed_types);
          }
        }
      }
    }

    $this->items = $items;
  }

  /**
   * Render each credit title.
   *
   * This will use tokens on the field if provided.
   */
  public function render_item($count, $item) {
    $items = [];
    $pattern = $this->options['course_credit_types_format'];
    if (!$pattern) {
      $pattern = '[title]';
    }

    foreach ($item as $row) {
      $credit = [];
      $credit['#markup'] = @strtr($pattern, $row);

      if (!empty($row['sub_credits'])) {
        $credit['children'] = [];
        foreach ($row['sub_credits'] as $subrow) {
          $type = $subrow['[type]'][0]['target_id'];
          $credit['children'][] = [
            '#markup' => @strtr($pattern, $subrow),
            '#wrapper_attributes' => [
              'class' => array('course-credit-' . Html::cleanCssIdentifier($type), 'course-child-credit'),
            ],
          ];
        }
      }

      $items[] = $credit;

    }

    $build = [
      '#theme' => 'item_list',
      '#items' => $items,
    ];

    $output = \Drupal::service('renderer')->render($build);
    return $output;

  }

  /**
   * Builds replacement values for the views field.
   *
   * @param \Drupal\course_credit\Entity\CourseCredit $course_credit
   * @param string $ctype The credit type machine name.
   * @param array $allowed_types An array of the allowed credit types.
   *
   * @return array
   */
  protected function buildCreditValues(CourseCredit $course_credit, $ctype, $allowed_types) {
    $build = [];
    $cc_props = $course_credit->toArray();

    $max = $cc_props['max'][0]['value'];
    $min = $cc_props['min'][0]['value'];

    $build["[max-s]"] = Drupal::translation()->formatPlural($max, '', 's')->__toString();
    $build["[min-s]"] = Drupal::translation()->formatPlural($min, '', 's')->__toString();

    $build["[title]"] = $allowed_types[$ctype];

    foreach ($cc_props as $key => $value) {
      $key = "[{$key}]";
      if (isset($value[0]['value'])) {
        $value = $value[0]['value'];
      }
      $build[$key] = $value;
    }

    return $build;
  }

  /**
   * Returns a flattened array of all active credit on a course.
   *
   * @param \Drupal\course_credit\Entity\CourseCredit $course
   *
   * @return array
   */
  protected function getAllCredits(Course $course) {
    $credits_all = [];
    $parents = $course->get('credit')->referencedEntities();
    foreach ($parents as $parent) {
      $credits_all[] = $parent;
      $children = $parent->get('children')->referencedEntities();
      foreach ($children as $child) {
        $credits_all[] = $child;
      }
    }

    return $credits_all;
  }

}
