<?php

/**
 * Field handler to provide a list of credit titles.
 */
class views_handler_field_course_credit_node_active_types extends views_handler_field_prerender_list {

  /**
   * Working with node table, adding active credit types to it.
   */
  function construct() {
    parent::construct();
    $this->additional_fields['nid'] = array(
      'table' => 'node',
      'field' => 'nid',
    );
  }

  /**
   * Add fields, alias.
   */
  function query() {
    $this->add_additional_fields();
    $this->field_alias = $this->aliases['nid'];
  }

  /**
   * Get the active credit types on each of the nodes in this view.
   */
  function pre_render(&$values) {
    $nids = array();
    $nid_field = $this->field_alias;
    foreach ($values as $value) {
      $nids[] = $value->{$nid_field};
    }

    if (!empty($this->options['course_credit_types'])) {
      $selected = array_filter($this->options['course_credit_types']);
    }

    if (empty($selected)) {
      // All credit types.
      $selected = array_keys(course_credit_get_credit_types());

      if (empty($selected)) {
        // There are no credit types.
        return;
      }
    }

    if ($nids) {
      $query = new EntityFieldQuery();
      $query->entityCondition('entity_type', 'course_credit')
        ->propertyCondition('nid', $nids)
        ->propertyCondition('pid', NULL)
        ->propertyCondition('active', 1);

      if (!empty($this->options['course_credit_types_negate'])) {
        $query->propertyCondition('type', $selected, 'not in');
      }
      else {
        $query->propertyCondition('type', $selected);
      }

      $result = $query->execute();
      $course_credits = [];
      if (!empty($result['course_credit'])) {
        $course_credits = entity_load('course_credit', array_keys($result['course_credit']));
      }

      $items = [];
      $pids = [];
      foreach ($course_credits as $ccid => $course_credit) {
        foreach ($course_credit as $key => $value) {
          $key = "[{$key}]";
          $items[$course_credit->nid][0][$ccid][$key] = $value;

          $items[$course_credit->nid][0][$ccid]["[max-s]"] = format_plural($course_credit->max, '', 's');
          $items[$course_credit->nid][0][$ccid]["[min-s]"] = format_plural($course_credit->min, '', 's');

          if (!empty($course_credit->data['sub_credits'])) {
            foreach ($course_credit->data['sub_credits'] as $sub_credit) {
              if ($sub_credit->active) {
                foreach ($sub_credit as $key => $value) {
                  $key = "[{$key}]";
                  $items[$course_credit->nid][0][$ccid]['sub_credits'][$sub_credit->type][$key] = $value;

                  $items[$course_credit->nid][0][$ccid]['sub_credits'][$sub_credit->type]["[max-s]"] = format_plural($course_credit->max, '', 's');
                  $items[$course_credit->nid][0][$ccid]['sub_credits'][$sub_credit->type]["[min-s]"] = format_plural($course_credit->min, '', 's');
                }
              }
            }
          }
        }
      }

      $this->items = $items;
    }
  }

  /**
   * Render each credit title.
   *
   * This will use tokens on the field if provided.
   */
  function render_item($count, $item) {
    $pattern = $this->options['course_credit_types_format'];
    if (!$pattern) {
      $pattern = '[title]';
    }

    $out = [];
    foreach ($item as $ccid => $row) {
      $child_items = [];
      // In PHP < 7 this throws a notice.
      $credit = @strtr($pattern, $row);
      if (!empty($row['sub_credits'])) {
        foreach ($row['sub_credits'] as $subrow) {
          $child_items[] = array(
            'data' => @strtr($pattern, $subrow),
            'class' => array('course-credit-' . drupal_html_class($subrow['[type]']), 'course-child-credit-list'),
          );
        }
      }
      $out[$ccid]['data'] = $credit;
      $out[$ccid]['children'] = $child_items;
    }


    return theme('item_list', ['items' => $out, 'type' => 'ul', 'attributes' => ['class' => 'course-credit-list'], 'title' => NULL]);
  }

  /**
   * Declare options.
   */
  function option_definition() {
    $options = parent::option_definition();

    $options['course_credit_types'] = array('default' => array());
    $options['course_credit_negate'] = array('default' => FALSE);
    $options['course_credit_types_format'] = array('default' => '');

    return $options;
  }

  /**
   * Select credit types to show.
   */
  function options_form(&$form, &$form_state) {
    parent::options_form($form, $form_state);
    $types = course_credit_get_credit_types();

    if ($types) {
      $options = array();
      foreach ($types as $ctid => $type) {
        $options[$type->type] = $type->title;
      }
    }

    $form['course_credit_types'] = array(
      '#title' => 'Credit types',
      '#type' => 'checkboxes',
      '#options' => $options,
      '#description' => 'Select credit types to show instead of showing all types.',
      '#default_value' => $this->options['course_credit_types'] ? $this->options['course_credit_types'] : array(),
    );

    $form['course_credit_types_negate'] = array(
      '#title' => 'Negate',
      '#type' => 'checkbox',
      '#description' => 'Do not show selected types.',
      '#default_value' => $this->options['course_credit_types_negate'],
    );

    $form['course_credit_types_format'] = array(
      '#title' => 'Format',
      '#type' => 'textfield',
      '#description' => 'Specify a format to use other than the name of the active credit type. Possible tokens: [title] [type] [min] [max]. You can also use [min-s] or [max-s] for the "S" after "credit" based on the number of credits.',
      '#default_value' => $this->options['course_credit_types_format'],
    );
  }

}
