<?php

namespace Drupal\Tests\course_credit\Functional;

use const REQUEST_TIME;
use function count;
use function course_credit_get_credit_awarded;
use function course_get_course;
use function course_unenroll;
use function db_select;
use function entity_create;

/**
 * Test class for course credit.
 *
 * @group Course credit
 */
class CourseCreditTestCase extends CourseCreditTestBase {

  public static $modules = ['course_credit_test'];

  /**
   * Test claim functionality (min, max, increments).
   */
  function testCourseCreditClaiming() {
    $courseNode = $this->createCourseNode();
    // Create instances.
    $courseNode->course_credit['variable'] = array(
      'increments' => 0.25,
      'min' => 1,
      'max' => 5,
      'variable' => 1,
      'active' => 1,
    );
    $courseNode->course_credit['nonvariable'] = array(
      'increments' => 0,
      'min' => 0,
      'max' => 5,
      'variable' => 0,
      'active' => 1,
    );
    node_save($courseNode);
    $account = $this->drupalCreateUser();
    course_enroll($courseNode, $account);
    $this->drupalLogin($account);
    course_get_course($courseNode)->getTracker($account)->setOption('complete', 1)->save();

    // Check that nonvariable credit did not get awarded automatically.
    $awarded = course_credit_get_credit_awarded($courseNode->nid, $account->uid);
    $this->assertEqual(count($awarded), '0', 'User was not yet awarded variable credit.');

    $this->drupalGet("node/$courseNode->nid/course-credit-app");
    $this->assertText("You can claim 5.00 Non-variable.");
    $this->assertText("Variable can be claimed in 0.25 increments up to a maximum of 5.00.");

    // Variable - increments
    $this->drupalPost(NULL, array(
      'credits_claimed[variable][credits][amount]' => 1.26,
      ), t('Submit'));
    $this->assertText('Credits claimed must be in increments of 0.25');

    // Variable - max
    $this->drupalPost(NULL, array(
      'credits_claimed[variable][credits][amount]' => 44,
      ), t('Submit'));
    $this->assertText('Credits claimed exceed maximum of 5.00.');

    // Variable - min
    $this->drupalPost(NULL, array(
      'credits_claimed[variable][credits][amount]' => 0.5,
      ), t('Submit'));
    $this->assertText('Credits claimed below minimum of 1.00.');

    // Variable - agree
    $this->drupalPost(NULL, array(
      ), t('Submit'));
    $this->assertText('Please verify your statement.');

    // Credit award
    $this->drupalPost(NULL, array(
      'credits_claimed[variable][credits][amount]' => 2.5,
      'agreement' => 1,
      ), t('Submit'));
    $awarded = course_credit_get_credit_awarded($courseNode->nid, $account->uid);
    $this->assertEqual(count($awarded), '2', 'User was awarded 2 credit types.');
    $this->assertEqual($awarded['variable']->credits, '2.5', 'Variable credit amount is correct.');
    $this->assertEqual($awarded['nonvariable']->credits, '5', 'Non-variable credit amount is correct.');

    course_unenroll($courseNode, $account);
    $awarded = course_credit_get_credit_awarded($courseNode->nid, $account->uid);
    $this->assertEqual(count($awarded), '0', 'User has 0 credit types.');
  }

  /**
   * Test expiration functionality.
   */
  function testCourseCreditExpiration() {
    $account = $this->drupalCreateUser();
    $this->drupalLogin($account);

    // Date
    $courseNode = $this->createCourseNode();
    $faketime = REQUEST_TIME + rand(86400, 300000);
    $credit_type_instance = array(
      'nid' => $courseNode->nid,
      'type' => 'nonvariable',
      'increments' => 0,
      'min' => 0,
      'max' => 5,
      'variable' => 0,
      'active' => 1,
      'expiration_type' => 'date',
      'expiration_date' => $faketime,
    );
    course_credit_save_course_credit_type_instance($credit_type_instance);
    course_get_course($courseNode)->getTracker($account)->setOption('complete', 1)->save();
    $this->drupalGet("node/$courseNode->nid/course-credit-app");
    $awarded = course_credit_get_credit_awarded($courseNode->nid, $account->uid);
    $this->assertEqual($awarded['nonvariable']->expiration, $faketime, 'Expiration date is correct.');

    // Date
    $courseNode = $this->createCourseNode();
    $credit_type_instance = array(
      'nid' => $courseNode->nid,
      'type' => 'nonvariable',
      'increments' => 0,
      'min' => 0,
      'max' => 5,
      'variable' => 0,
      'active' => 1,
      'expiration_type' => 'calendar_year',
      'expiration_offset' => 86400 * 2,
    );
    $expiration_calyear_shouldbe = mktime(0, 0, 0, 1, 1, date('Y') + 1) - 1 + 86400 * 2;
    course_credit_save_course_credit_type_instance($credit_type_instance);
    course_get_course($courseNode)->getTracker($account)->setOption('complete', 1)->save();
    $this->drupalGet("node/$courseNode->nid/course-credit-app");
    $awarded = course_credit_get_credit_awarded($courseNode->nid, $account->uid);
    $this->assertEqual($awarded['nonvariable']->expiration, $expiration_calyear_shouldbe, 'Calendar year expiration date is correct.');

    // Credit awarded
    $courseNode = $this->createCourseNode();
    $credit_type_instance = array(
      'nid' => $courseNode->nid,
      'type' => 'nonvariable',
      'increments' => 0,
      'min' => 0,
      'max' => 5,
      'variable' => 0,
      'active' => 1,
      'expiration_type' => 'credit_awarded',
      'expiration_offset' => 86400 * 2,
    );
    course_credit_save_course_credit_type_instance($credit_type_instance);
    course_get_course($courseNode)->getTracker($account)->setOption('complete', 1)->save();
    $this->drupalGet("node/$courseNode->nid/course-credit-app");
    $awarded = course_credit_get_credit_awarded($courseNode->nid, $account->uid);
    $this->assertEqual($awarded['nonvariable']->expiration, $awarded['nonvariable']->date + 86400 * 2, 'Credit awarded year expiration date is correct.');
  }

  /**
   * Test course credit awarded revisioning.
   */
  public function testCourseCreditAwardedRevisions() {
    // Create a course with the both credit types active.
    $courseNode = $this->createCourseNode();
    $courseNode->course_credit['variable'] = array(
      'increments' => 0.25,
      'min' => 1,
      'max' => 5,
      'variable' => 1,
      'active' => 1,
    );
    $courseNode->course_credit['nonvariable'] = array(
      'increments' => 0,
      'min' => 0,
      'max' => 5,
      'variable' => 0,
      'active' => 1,
    );
    node_save($courseNode);

    // Save credit awarded.
    course_enroll($courseNode, $this->student_user);
    $cca = entity_create('course_credit_awarded', array());
    $cca->nid = $courseNode->nid;
    $cca->uid = $this->student_user->uid;
    $cca->type = 'nonvariable';
    $cca->credits = 10.00;
    entity_save('course_credit_awarded', $cca);
    $this->drupalLogin($this->student_user);
    $this->drupalGet("node/$courseNode->nid/takecourse");
    $this->clickLink(t('Credit'));
    $this->assertText('You were awarded 10.00');

    // Create a new revision.
    $cca->credits = 5.00;
    $cca->is_new_revision = TRUE;
    entity_save('course_credit_awarded', $cca);
    $this->assertEqual(2, count(db_select('course_credit_awarded_revision', 'ccar')->fields('ccar')->execute()->fetchAll()), 'Found 2 revisions.');

    // Ensure the current revision is used.
    $this->drupalGet("node/$courseNode->nid/takecourse");
    $this->clickLink(t('Credit'));
    $this->assertText('You were awarded 5.00');
  }

  /**
   * Test opt/in out of credit settings.
   */
  function testOptInOutCredit() {
    // Create a course with the both credit types active.
    $courseNode = $this->createCourseNode();
    $courseNode->course_credit['variable'] = array(
      'increments' => 0.25,
      'min' => 1,
      'max' => 5,
      'variable' => 1,
      'active' => 1,
    );
    $courseNode->course_credit['nonvariable'] = array(
      'increments' => 0,
      'min' => 0,
      'max' => 5,
      'variable' => 0,
      'active' => 1,
    );
    node_save($courseNode);

    $account = $this->drupalCreateUser();
    course_enroll($courseNode, $account);
    $this->drupalLogin($account);

    variable_set('course_credit_select_all', 1);
    $this->drupalGet("node/$courseNode->nid/takecourse");
    $this->clickLink(t('Credit'));
    $this->assertFieldById('edit-credits-claimed-variable-claim', 1);
    $this->assertFieldById('edit-credits-claimed-nonvariable-claim', 1);

    variable_set('course_credit_select_all', 0);
    $this->drupalGet("node/$courseNode->nid/takecourse");
    $this->clickLink(t('Credit'));
    $this->assertFieldById('edit-credits-claimed-variable-claim', 0);
    $this->assertFieldById('edit-credits-claimed-nonvariable-claim', 0);
  }

  /**
   * Test skipping credit.
   */
  function testSkipClaim() {
    // Set for opt-in.
    variable_set('course_credit_select_all', 0);
    variable_set('course_credit_allow_skip', 1);

    // Create a course with the both credit types active.
    $courseNode = $this->createCourseNode();
    $courseNode->course_credit['variable'] = array(
      'increments' => 0.25,
      'min' => 1,
      'max' => 5,
      'variable' => 1,
      'active' => 1,
    );
    $courseNode->course_credit['nonvariable'] = array(
      'increments' => 0,
      'min' => 0,
      'max' => 5,
      'variable' => 0,
      'active' => 1,
    );
    node_save($courseNode);

    // Confirm the user has to indicate they are not claiming credit.
    course_enroll($courseNode, $this->student_user);
    $this->drupalLogin($this->student_user);
    $this->drupalGet("node/$courseNode->nid/takecourse");
    $this->clickLink(t('Credit'));
    $this->drupalPost(NULL, array(), t('Submit'));
    $this->assertText(t('Please indicate that you are not claiming any credit.'));
    $this->drupalPost(NULL, array('not_claiming' => 1), t('Submit'));
    $this->assertNoText(t('Please indicate that you are not claiming any credit.'));
  }

  /**
   * Test selecting a credit type.
   */
  function testCourseCreditSelect() {
    $courseNode = $this->createCourseNode();
    // Create instances.
    $courseNode->course_credit['variable'] = array(
      'increments' => 0.25,
      'min' => 1,
      'max' => 5,
      'variable' => 1,
      'active' => 1,
    );
    // We need two variable types here.
    $courseNode->course_credit['nonvariable'] = array(
      'increments' => 0.25,
      'min' => 1,
      'max' => 5,
      'variable' => 1,
      'active' => 1,
    );
    node_save($courseNode);
    $account = $this->drupalCreateUser();
    course_enroll($courseNode, $account);
    $this->drupalLogin($account);
    course_get_course($courseNode)->getTracker($account)->setOption('complete', 1)->save();

    $this->drupalGet("node/$courseNode->nid/course-credit-app");

    // Test claiming one, but not both types.
    $this->drupalPost(NULL, array(
      'credits_claimed[variable][claim]' => 1,
      'credits_claimed[variable][credits][amount]' => 1.25,
      'credits_claimed[nonvariable][claim]' => FALSE,
      'agreement' => 1,
      ), t('Submit'));
    $this->assertText('Awarded credit.');
  }

}
